@@ -32,17 +32,22 @@
 
 /* a client may be in one of several states */
 enum esd_client_state {
-    ESD_NEEDS_VALIDATION,	/* need to validate this client/request */
-    ESD_NEEDS_ENDCHECK, 	/* need to validate this client/request */
     ESD_STREAMING_DATA,		/* data from here on is streamed data */
     ESD_CACHING_SAMPLE,		/* midway through caching a sample */
     ESD_NEEDS_REQDATA,		/* more data needed to complere request */
     ESD_NEXT_REQUEST,		/* proceed to next request */
-
     ESD_CLIENT_STATE_MAX	/* place holder */
 };
 typedef int esd_client_state_t;
 
+/* mix functions control how the player data is combined */
+typedef int mix_func_t( void *player, int length );
+typedef int translate_func_t( void *dest_buf, int dest_len, 
+			      int dest_rate, esd_format_t dest_format, 
+			      void *source_data, int src_len, 
+			      int src_rate, esd_format_t src_format );
+/* NOTE: in heavy flux! */
+
 /* a client is what contacts the server, and makes requests of daemon */
 typedef struct esd_client {
     struct esd_client *next; 	/* it's a list, eh? link 'em */
@@ -54,14 +59,14 @@ typedef struct esd_client {
     struct sockaddr_in source;	/* data maintained about source */
 
     int swap_byte_order;	/* for big/little endian compatibility */
-    /* TODO: read_data() should then swap all the bytes */
-    /* before the mixing subsystem gets a hold of it. */
+    octet proto_data[ 1024 ];	/* hold the protocol request data */
+    int proto_data_length;	/* how much request data we have */
 } esd_client_t;
 
 /* a player is what produces data for a sound */
 typedef struct esd_player {
     struct esd_player *next;	/* point to next player in list */
-    void *parent;		/* the client or sample that spawned player */
+    void *parent;		/* client or sample that spawned player */
 
     esd_format_t format;	/* magic int with the format info */
     int rate;			/* sample rate */
@@ -76,6 +81,9 @@ typedef struct esd_player {
     /* time_t last_read; */	/* timeout for streams, not used */
     int last_pos;		/* track read position for samples */
     char name[ ESD_NAME_MAX ];	/* name of stream for remote control */
+
+    mix_func_t *mix_func;	/* mixes data into the combined buffer */
+    translate_func_t *translate_func;	/* copies data between players */
 } esd_player_t;
 
 /* TODO?: typedef esd_player_t esd_recorder_t, and monitor? */
@@ -104,52 +112,74 @@ typedef struct esd_sample {
 /* server function prototypes */
 
 /* esd.c - global variables */
+extern int esd_is_owned;
+extern int esd_is_locked;
+extern char esd_owner_key[ESD_KEY_LEN];
+
 extern int esd_on_standby;
 extern int esdbg_trace;
 extern int esdbg_comms;
+extern int esdbg_mixer;
 
 extern int esd_buf_size_octets;
 extern int esd_buf_size_samples;
 extern int esd_sample_size;
 
+extern int esd_autostandby_secs;
+extern time_t esd_last_activity;
+extern int esd_on_autostandby;
+
+int esd_server_standby(void);
+int esd_server_resume(void);
+
 /* clients.c - manage the client connections */
 extern esd_client_t *esd_clients_list;
-void dump_clients();
+void dump_clients(void);
 void add_new_client( esd_client_t *new_client );
 void erase_client( esd_client_t *client );
 
-void clear_auth( int signum ); /* TODO: sig_clear_auth ? */
 int get_new_clients( int listen );
 int wait_for_clients_and_data( int listen );
 
+/* filter.c - things with which to handle filters */
+extern esd_player_t *esd_filter_list;
+extern translate_func_t *esd_first_filter_func;
+
+void erase_filter( esd_player_t *filter );
+
 /* proto.c - deal with client protocol requests */
-int poll_client_requests();
+extern int esd_forced_standby;
+
+int poll_client_requests(void);
 
 /* players.c - manage the players, recorder, and monitor */
 extern esd_player_t *esd_players_list;
 extern esd_player_t *esd_recorder;
 extern esd_player_t *esd_monitor_list;
 
-/* filter.c - things with which to handle filters */
-extern esd_player_t *esd_filter_list;
-void erase_filter( esd_player_t *filter );
-
-void dump_players();
+void dump_players(void);
 void add_player( esd_player_t *player );
 void erase_player( esd_player_t *player );
+void free_player( esd_player_t *player );
+void erase_monitor( esd_player_t *monitor );
+
 esd_player_t *new_stream_player( esd_client_t *client );
 esd_player_t *new_sample_player( int id, int loop );
 
+int write_player( esd_player_t *player, void *src_buffer, int src_length, 
+		  int src_rate, int src_format );
 int read_player( esd_player_t *player );
-void recorder_write();
+int recorder_write( void *buffer, int length );
 void monitor_write( void *output_buffer, int length );
 
 /* samples.c - manage the players, recorder, and monitor */
 extern esd_sample_t *esd_samples_list;
+extern int esd_playing_samples;
+extern int esd_next_sample_id;
 
-void dump_samples();
+void dump_samples(void);
 void add_sample( esd_sample_t *sample );
-void erase_sample( int id );
+void erase_sample( int id, int force );
 esd_sample_t *new_sample( esd_client_t *client );
 esd_sample_t *find_caching_sample( esd_client_t *client );
 int read_sample( esd_sample_t *sample );
@@ -157,24 +187,21 @@ int play_sample( int sample_id, int loop );
 int stop_sample( int sample_id );
 
 /* mix.c - deal with mixing signals, and format conversion */
-int mix_and_copy( void *dest_buf, int dest_len, 
-	int dest_rate, esd_format_t dest_format, 
-	void *source_data, int src_len, 
-	int src_rate, esd_format_t src_format );
-int mix_from_stereo_16s( void *dest_buf, int dest_len, 
-	int dest_rate, esd_format_t dest_format, 
-	signed short *source_data_ss, int src_len, int src_rate );
-int mix_from_stereo_8u( void *dest_buf, int dest_len, 
-	int dest_rate, esd_format_t dest_format, 
-	unsigned char *source_data_uc, int src_len, int src_rate );
-int mix_from_mono_16s( void *dest_buf, int dest_len, 
-	int dest_rate, esd_format_t dest_format, 
-	signed short *source_data_ss, int src_len, int src_rate );
-int mix_from_mono_8u( void *dest_buf, int dest_len, 
-	int dest_rate, esd_format_t dest_format, 
-	unsigned char *source_data_uc, int src_len, int src_rate );
-int mix_players_16s( void *mixed, int length );
-int mix_players_8s( void *mixed, int length );
+mix_func_t *get_mix_func( esd_player_t *player );
+translate_func_t *get_translate_func( esd_format_t src_fmt, int src_rate,
+				      esd_format_t dst_fmt, int dst_rate );
+
+int refresh_mix_funcs();
+int mix_players( void *mixed, int length );
+
+/*******************************************************************/
+/* filter.c */
+int filter_write( void *buffer, int size, esd_format_t format, int rate );
+
+/*******************************************************************/
+/* esdlib.c */
+int esd_set_socket_buffers( int sock, int src_format, 
+			    int src_rate, int base_rate );
 
 /*******************************************************************/
 /* evil evil macros */
@@ -192,28 +219,42 @@ int mix_players_8s( void *mixed, int length );
 #define maybe_swap_32(c,x) \
 	( (c) ? (swap_endian_32( (x) )) : (x) )
 
+#define ESD_DEFAULT_AUTOSTANDBY_SECS 5
+
 /* evil macros for debugging protocol */
 
 #ifdef ESDBG /* expand debug macros to yield diagnostic information */
 
 #define ESDBG_TRACE(x) \
 	do { \
-	    if ( esdbg_trace ) \
+	    if ( esdbg_trace ) { \
+		printf( ":trace: [%12s,%5d] \t", __FILE__, __LINE__ ); \
     		x; \
+	    } \
 	} while( 0 );
 
 #define ESDBG_COMMS(x) \
 	do { \
-	    if ( esdbg_comms ) \
+	    if ( esdbg_comms ) { \
+		printf( ":comms: [%12s,%5d] \t", __FILE__, __LINE__ ); \
     		x; \
+	    } \
+	} while( 0 );
+
+#define ESDBG_MIXER(x) \
+	do { \
+	    if ( esdbg_mixer ) { \
+		printf( "+mixer+ [%12s,%5d] \t", __FILE__, __LINE__ ); \
+    		x; \
+	    } \
 	} while( 0 );
 
 #define ESD_READ_INT(s,a,l,r,d) \
 	do { \
-	    int esd_ri; \
+	    unsigned int esd_ri; \
     	    r = read( s, a, l ); \
     	    if ( esdbg_comms ) { \
-		printf( "---> rd [%s,%d] \t%-10s: %2d, %4d = %4d  (%d)     (", \
+		printf( "---> rd [%12s,%5d] \t%-10s: %2d, %4d = %4d  (%d)     (", \
 			__FILE__, __LINE__, d, s, l, r, *a ); \
     		for ( esd_ri = 0 ; esd_ri < sizeof(int) ; esd_ri++ ) \
     		    printf( "0x%02x ", *(((octet*)a)+esd_ri) ); \
@@ -223,10 +264,10 @@ int mix_players_8s( void *mixed, int length );
 
 #define ESD_READ_BIN(s,a,l,r,d) \
 	do { \
-	    int esd_rb; \
+	    unsigned int esd_rb; \
     	    r = read( s, a, l ); \
     	    if ( esdbg_comms ) { \
-		printf( "---> rd [%s,%d] \t%-10s: %2d, %4d = %4d  (", \
+		printf( "---> rd [%12s,%5d] \t%-10s: %2d, %4d = %4d  (", \
 			__FILE__, __LINE__, d, s, l, r ); \
     		for ( esd_rb = 0 ; esd_rb < 8 ; esd_rb++ ) \
     		    printf( "0x%02x ", *(((octet*)a)+esd_rb) ); \
@@ -236,10 +277,10 @@ int mix_players_8s( void *mixed, int length );
 
 #define ESD_WRITE_INT(s,a,l,r,d) \
 	do { \
-	    int esd_wi; \
+	    unsigned int esd_wi; \
     	    r = write( s, a, l ); \
     	    if ( esdbg_comms ) { \
-		printf( "<--- wr [%s,%d] \t%-10s: %2d, %4d = %4d  (%d)     (", \
+		printf( "<--- wr [%12s,%5d] \t%-10s: %2d, %4d = %4d  (%d)     (", \
 			__FILE__, __LINE__, d, s, l, r, *a ); \
     		for ( esd_wi = 0 ; esd_wi < sizeof(int) ; esd_wi++ ) \
     		    printf( "0x%02x ", *(((octet*)a)+esd_wi) ); \
@@ -249,10 +290,10 @@ int mix_players_8s( void *mixed, int length );
 
 #define ESD_WRITE_BIN(s,a,l,r,d) \
 	do { \
-	    int esd_wb; \
+	    unsigned int esd_wb; \
     	    r = write( s, a, l ); \
     	    if ( esdbg_comms ) { \
-		printf( "<--- wr [%s,%d] \t%-10s: %2d, %4d = %4d  (", \
+		printf( "<--- wr [%12s,%5d] \t%-10s: %2d, %4d = %4d  (", \
 			__FILE__, __LINE__, d, s, l, r ); \
     		for ( esd_wb = 0 ; esd_wb < 8 ; esd_wb++ ) \
     		    printf( "0x%02X ", *(((octet*)a)+esd_wb) ); \
@@ -264,26 +305,12 @@ int mix_players_8s( void *mixed, int length );
 
 #define ESDBG_TRACE(x)
 #define ESDBG_COMMS(x)
+#define ESDBG_MIXER(x)
 
-#define ESD_READ_INT(s,a,l,r,d) \
-	do { \
-    	    r = read( s, a, l ); \
-        } while ( 0 );
-
-#define ESD_READ_BIN(s,a,l,r,d) \
-	do { \
-    	    r = read( s, a, l ); \
-        } while ( 0 );
-
-#define ESD_WRITE_INT(s,a,l,r,d) \
-	do { \
-    	    r = write( s, a, l ); \
-        } while ( 0 );
-
-#define ESD_WRITE_BIN(s,a,l,r,d) \
-	do { \
-    	    r = write( s, a, l ); \
-        } while ( 0 );
+#define ESD_READ_INT(s,a,l,r,d)	 r = read( s, a, l );
+#define ESD_READ_BIN(s,a,l,r,d)  r = read( s, a, l );
+#define ESD_WRITE_INT(s,a,l,r,d) r = write( s, a, l );
+#define ESD_WRITE_BIN(s,a,l,r,d) r = write( s, a, l );
 
 #endif /* #ifdef ESDBG */
 
