@@ -8,18 +8,38 @@
 #include <unistd.h>
 #endif
 
+/*******************************************************************/
+/* esd.c - prototypes */
+void set_audio_buffer( void *buf, esd_format_t format, int magl, int magr, 
+		int freq, int speed, int length, long offset );
+void clean_exit( int signum );
+void reset_signal( int signum );
+void reset_daemon( int signum );
+int open_listen_socket( int port );
+
 /*******************************************************************/
 /* globals */
-int esd_on_standby = 0;	/* set to 1 to route all audio to /dev/null */
+
+int esd_is_owned = 0;		/* start unowned, first client claims it */
+int esd_is_locked = 1;		/* if owned, will prohibit foreign sources */
+char esd_owner_key[ESD_KEY_LEN]; /* the key that locks the daemon */
+
+int esd_on_standby = 0;		/* set to route ignore incoming audio data */
 int esdbg_trace = 0;		/* show warm fuzzy debug messages */
 int esdbg_comms = 0;		/* show protocol level debug messages */
+int esdbg_mixer = 0;		/* show mixer engine debug messages */
 
 int esd_buf_size_octets = 0; 	/* size of audio buffer in bytes */
 int esd_buf_size_samples = 0; 	/* size of audio buffer in samples */
 int esd_sample_size = 0;	/* size of sample in bytes */
 
 int esd_beeps = 1;		/* whether or not to beep on startup */
-int listen_socket = -1;		/* socket we're accepting connections on */
+int listen_socket = -1;		/* socket to accept connections on */
+
+int esd_autostandby_secs = -1; 	/* timeout to release audio device, disabled <0 */
+time_t esd_last_activity = 0;	/* seconds since last activity */
+int esd_on_autostandby = 0;	/* set when auto paused for auto reawaken */
+
 
 /*******************************************************************/
 /* just to create the startup tones for the fun of it */
@@ -66,6 +86,46 @@ void set_audio_buffer( void *buf, esd_format_t format,
 
 /*******************************************************************/
 /* to properly handle signals */
+
+void reset_daemon( int signum )
+{
+    int tumbler;
+
+    ESDBG_TRACE( 
+	printf( "(ca) resetting sound daemon on signal %d\n",
+		signum ); );
+
+    /* reset the access rights */
+    esd_is_owned = 0;
+    esd_is_locked = 1;
+
+    /* scramble the stored key */
+    srand( time(NULL) );
+    for ( tumbler = 0 ; tumbler < ESD_KEY_LEN ; tumbler++ ) {
+	esd_owner_key[ tumbler ] = rand() % 256;
+    }
+
+    /* close the clients */
+    while ( esd_clients_list != NULL )
+    {
+	erase_client( esd_clients_list );
+    }
+
+    /* free samples */
+    while ( esd_samples_list != NULL )
+    {
+	erase_sample( esd_samples_list->sample_id, 1 );
+	/* TODO: kill_sample, so it stops playing */
+	/* a looping sample will get stuck */
+    }
+
+    /* reset next sample id */
+    esd_next_sample_id = 1;
+
+    /* reset signal handler, if not called from a signal, no effect */
+    signal( SIGHUP, reset_daemon );
+}
+
 void clean_exit(int signum) {
     /* just clean up as best we can and terminate from here */
     esd_client_t * client = esd_clients_list;
@@ -130,6 +190,11 @@ int open_listen_socket( int port )
 		lin.l_linger);
 	return( -1 );
     }
+    {
+      int n = 1;
+      setsockopt(socket_listen, SOL_SOCKET, SO_REUSEADDR, &n, sizeof(n));
+      /* if it fails, so what */
+    }
 
     /* set the listening information */
     socket_addr.sin_family = AF_INET;
@@ -139,8 +204,7 @@ int open_listen_socket( int port )
 	       (struct sockaddr *) &socket_addr,
 	       sizeof(struct sockaddr_in) ) < 0 )
     {
-	fprintf(stderr,"Unable to bind port %d\n", 
-		socket_addr.sin_port );
+	fprintf(stderr,"Unable to bind port %d\n", port );
 	exit(1);
     }
     if (listen(socket_listen,16)<0)
@@ -152,6 +216,49 @@ int open_listen_socket( int port )
     return socket_listen;
 }
 
+/*******************************************************************/
+/* daemon eats sound data, without playing anything, return boolean ok */
+int esd_server_standby(void)
+{
+    int ok = 1;
+
+    /* only bother if we're not already on standby */
+    if ( !esd_on_standby ) {
+	ESDBG_TRACE( printf( "setting sound daemon to standby\n" ); );
+	
+	/* TODO: close down any recorders, too */
+	esd_on_standby = 1;
+	esd_audio_close();
+    }	
+
+    return ok;
+}
+
+/*******************************************************************/
+/* daemon goes back to playing sound data, returns boolean ok */
+int esd_server_resume(void)
+{
+    int ok = 1;
+
+    /* only bother if we're already on standby */
+    if ( esd_on_standby ) {
+	
+	ESDBG_TRACE( printf( "resuming sound daemon\n" ); );
+	
+	/* reclaim the audio device */
+	if ( esd_audio_open() < 0 ) {
+	    /* device was busy or something, return error, try  later */
+	    ok = 0;
+	} else {
+	    /* turn ourselves back on */
+	    esd_on_standby = 0;
+	    esd_on_autostandby = 0;
+	}
+    }
+
+    return ok;
+}
+
 /*******************************************************************/
 int main ( int argc, char *argv[] )
 {
@@ -164,8 +271,6 @@ int main ( int argc, char *argv[] )
 
     void *output_buffer = NULL;
 
-    int do_sleep = 0;
-
     /* begin test scaffolding parameters */
     /* int format = AFMT_U8; AFMT_S16_LE; */
     /* int stereo = 0; */     /* 0=mono, 1=stereo */
@@ -176,17 +281,25 @@ int main ( int argc, char *argv[] )
     int default_format = ESD_BITS16 | ESD_STEREO;
     /* end test scaffolding parameters */
 
-    /* start the initializatin process */
-    printf( "initializing...\n" );
-
     /* parse the command line args */
     for ( arg = 1 ; arg < argc ; arg++ ) {
-	if ( !strcmp( argv[ arg ], "-port" ) ) {
+	if ( !strcmp( argv[ arg ], "-d" ) ) {
+	    if ( ++arg != argc ) {
+		esd_audio_device = argv[ arg ];
+		if ( !esd_audio_device ) {
+		    esd_port = ESD_DEFAULT_PORT;
+		    fprintf( stderr, "- could not read device: %s\n", 
+			     argv[ arg ] );
+		}
+		fprintf( stderr, "- using device %s\n", 
+			 esd_audio_device );
+	    }
+	} else if ( !strcmp( argv[ arg ], "-port" ) ) {
 	    if ( ++arg != argc ) {
 		esd_port = atoi( argv[ arg ] );
 		if ( !esd_port ) {
 		    esd_port = ESD_DEFAULT_PORT;
-		    fprintf( stderr, "- could not determine port: %s\n", 
+		    fprintf( stderr, "- could not read port: %s\n", 
 			     argv[ arg ] );
 		}
 		fprintf( stderr, "- accepting connections on port %d\n", 
@@ -200,26 +313,59 @@ int main ( int argc, char *argv[] )
 		default_rate = atoi( argv[ arg ] );
 		if ( !default_rate ) {
 		    default_rate = ESD_DEFAULT_RATE;
-		    fprintf( stderr, "- could not determine rate: %s\n", 
+		    fprintf( stderr, "- could not read rate: %s\n", 
 			     argv[ arg ] );
 		}
-		fprintf( stderr, "- server_format: sample rate = %d Hz\n", 
+		fprintf( stderr, "- server format: sample rate = %d Hz\n", 
 			 default_rate );
 	    }
+	} else if ( !strcmp( argv[ arg ], "-as" ) ) {
+	    if ( ++arg != argc ) {
+		esd_autostandby_secs = atoi( argv[ arg ] );
+		if ( !esd_autostandby_secs ) {
+		    esd_autostandby_secs = ESD_DEFAULT_AUTOSTANDBY_SECS;
+		    fprintf( stderr, "- could not read autostandby timeout: %s\n", 
+			     argv[ arg ] );
+		}
+		fprintf( stderr, "- autostandby timeout: %d seconds\n", 
+			 esd_autostandby_secs );
+	    }
+#ifdef ESDBG
 	} else if ( !strcmp( argv[ arg ], "-vt" ) ) {
 	    esdbg_trace = 1;
 	    fprintf( stderr, "- enabling trace diagnostic info\n" );
 	} else if ( !strcmp( argv[ arg ], "-vc" ) ) {
 	    esdbg_comms = 1;
 	    fprintf( stderr, "- enabling comms diagnostic info\n" );
+	} else if ( !strcmp( argv[ arg ], "-vm" ) ) {
+	    esdbg_mixer = 1;
+	    fprintf( stderr, "- enabling mixer diagnostic info\n" );
+#endif
 	} else if ( !strcmp( argv[ arg ], "-nobeeps" ) ) {
 	    esd_beeps = 0;
 	    fprintf( stderr, "- disabling startup beeps\n" );
+	} else if ( !strcmp( argv[ arg ], "-h" ) ) {
+	    fprintf( stderr, "Usage: esd [options]\n\n" );
+	    fprintf( stderr, "  -d DEVICE   force esd to use sound device DEVICE\n" );
+	    fprintf( stderr, "  -b          run server in 8 bit sound mode\n" );
+	    fprintf( stderr, "  -r RATE     run server at sample rate of RATE\n" );
+	    fprintf( stderr, "  -as SECS    free audio device after SECS of inactivity\n" );
+#ifdef ESDBG
+	    fprintf( stderr, "  -vt         enable trace diagnostic info\n" );
+	    fprintf( stderr, "  -vc         enable comms diagnostic info\n" );
+	    fprintf( stderr, "  -vm         enable mixer diagnostic info\n" );
+#endif
+	    fprintf( stderr, "  -port PORT  listen for connections at PORT\n" );
+	    fprintf( stderr, "\nPossible devices are:  %s\n", esd_audio_devices() );
+	    exit( 0 );
 	} else {
 	    fprintf( stderr, "unrecognized option: %s\n", argv[ arg ] );
 	}
     }
 
+    /* start the initializatin process */
+    printf( "initializing...\n" );
+
     /* set the data size parameters */
     esd_audio_format = default_format;
     esd_audio_rate = default_rate;
@@ -251,7 +397,7 @@ int main ( int argc, char *argv[] )
     signal( SIGINT, clean_exit );	/* for ^C */
     signal( SIGTERM, clean_exit );	/* for default kill */
     signal( SIGPIPE, reset_signal );	/* for closed rec/mon clients */
-    signal( SIGHUP, clear_auth );	/* kill -HUP clear ownership */
+    signal( SIGHUP, reset_daemon );	/* kill -HUP clear ownership */
 
     /* send some sine waves just to check the sound connection */
     i = 0;
@@ -287,8 +433,15 @@ int main ( int argc, char *argv[] )
 	poll_client_requests();
 
 	/* mix new requests, and output to device */
-	length = mix_players_16s( output_buffer, esd_buf_size_octets );
+	refresh_mix_funcs(); /* TODO: set a flag to cue when to do this */
+	length = mix_players( output_buffer, esd_buf_size_octets );
 	
+	/* awaken if on autostandby and doing anything */
+	if ( esd_on_autostandby && length && !esd_forced_standby ) {
+	    ESDBG_TRACE( printf( "stuff to play, waking up.\n" ); );
+	    esd_server_resume();
+	}
+
 	/* we handle this even when length == 0 because a filter could have
 	 * closed, and we don't want to eat the processor if one did.. */
 	if ( esd_filter_list && !esd_on_standby ) {
@@ -297,7 +450,7 @@ int main ( int argc, char *argv[] )
 	}
 	
 	if ( length > 0 /* || esd_monitor */ ) {
-	    do_sleep = 0;
+	    /* do_sleep = 0; */ 
 	    if ( !esd_on_standby ) {
 		/* standby check goes in here, so esd will eat sound data */
 		/* TODO: eat a round of data with a better algorithm */
@@ -305,18 +458,26 @@ int main ( int argc, char *argv[] )
 		/* TODO: on monitor, why isn't this a buffer of zeroes? */
 		/* esd_audio_write( output_buffer, esd_buf_size_octets ); */
 		esd_audio_write( output_buffer, length );
-		esd_audio_flush();
+		/* esd_audio_flush(); */ /* this is overkill */
+		esd_last_activity = time( NULL );
 	    }
 	} else {
+	    /* should be pausing just fine within wait_for_clients_and_data */
+	    /* if so, this isn't really needed */
+
 	    /* be very quiet, and wait for a wabbit to come along */
+#if 0
+	    if ( !do_sleep ) { ESDBG_TRACE( printf( "pausing in esd.c\n" ); ); }
 	    do_sleep = 1;
 	    esd_audio_pause();
+#endif
 	}
 
 	/* if someone's monitoring the sound stream, send them data */
 	/* mix_players, above, forces buffer to zero if no players */
 	/* this clears out any leftovers from recording, below */
 	if ( esd_monitor_list && !esd_on_standby && length ) {
+	/* if ( esd_monitor_list && !esd_on_standby ) {  */
 	    monitor_write( output_buffer, length );
 	}
 
@@ -324,30 +485,12 @@ int main ( int argc, char *argv[] )
 	if ( esd_recorder && !esd_on_standby ) { 
 	    length = esd_audio_read( output_buffer, esd_buf_size_octets );
 	    if ( length ) {
-		if ( (esd_audio_format & ESD_MASK_BITS) == ESD_BITS16 ) {
-		    length = mix_from_stereo_16s( esd_recorder->data_buffer, 
-						  esd_recorder->buffer_length, 
-						  esd_recorder->rate, 
-						  esd_recorder->format, 
-						  output_buffer, 
-						  esd_buf_size_octets, 
-						  esd_audio_rate );
-		}
-		else {
-		    length = mix_from_stereo_8u( esd_recorder->data_buffer, 
-						 esd_recorder->buffer_length, 
-						 esd_recorder->rate, 
-						 esd_recorder->format, 
-						 output_buffer, 
-						 esd_buf_size_octets, 
-						 esd_audio_rate );
-		}
-		
-		recorder_write(); 
+		length = recorder_write( output_buffer, length ); 
+		esd_last_activity = time( NULL );
 	    }
 	}
 
-	if ( esd_on_standby || do_sleep ) {
+	if ( esd_on_standby ) {
 #ifdef HAVE_NANOSLEEP
 	    struct timespec restrain;
 	    restrain.tv_sec = 0;
@@ -366,12 +509,9 @@ int main ( int argc, char *argv[] )
 	    select( 0, 0, 0, 0, &restrain );
 #endif
 	}
-
     } /* while ( 1 ) */
 
-    /* how we'd get here, i have no idea */
-    esd_audio_close();
-    close( listen_socket );
-
+    /* how we'd get here, i have no idea, should only exit on signal */
+    clean_exit( -1 );
     exit( 0 );
 }
