@@ -1,5 +1,6 @@
 #ifndef ESD_H
 #define ESD_H
+#include <audiofile.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -22,14 +23,14 @@ extern "C" {
 
 /* a magic number to identify the relative endianness of a client */
 #define ESD_ENDIAN_KEY \
-	( ('E' << 24) + ('N' << 16) + ('D' << 8) + ('N') )
+	( (unsigned int) ( ('E' << 24) + ('N' << 16) + ('D' << 8) + ('N') ) )
 
 #define ESD_VOLUME_BASE (256)
 
 /*************************************/
 /* what can we do to/with the EsounD */
 enum esd_proto { 
-    ESD_PROTO_INVALID,      /* hold the zero value */
+    ESD_PROTO_CONNECT,      /* implied on inital client connection */
 
     /* pseudo "security" functionality */
     ESD_PROTO_LOCK,	    /* disable "foreign" client connections */
@@ -52,13 +53,13 @@ enum esd_proto {
     ESD_PROTO_STANDBY,	    /* release /dev/dsp and ignore all data */
     ESD_PROTO_RESUME,	    /* reclaim /dev/dsp and play sounds again */
 
-    /* move these to a more logical place when we're ready to break the protocol */
+    /* TODO: move these to a more logical place. NOTE: will break the protocol */
     ESD_PROTO_SAMPLE_GETID, /* get the ID for an already-cached sample */
     ESD_PROTO_STREAM_FILT,  /* filter mixed buffer output as a stream */
 
     /* esd remote management */
-    ESD_PROTO_SERVER_INFO,  /* get server information (ver, sample rate, format) */
-    ESD_PROTO_ALL_INFO,     /* get all information (server info, players, samples) */
+    ESD_PROTO_SERVER_INFO,  /* get server info (ver, sample rate, format) */
+    ESD_PROTO_ALL_INFO,     /* get all info (server info, players, samples) */
     ESD_PROTO_SUBSCRIBE,    /* track new and removed players and samples */
     ESD_PROTO_UNSUBSCRIBE,  /* stop tracking updates */
 
@@ -66,8 +67,12 @@ enum esd_proto {
     ESD_PROTO_STREAM_PAN,   /* set stream panning */
     ESD_PROTO_SAMPLE_PAN,   /* set default sample panning */
 
+    /* esd status */
+    ESD_PROTO_STANDBY_MODE, /* see if server is in standby, autostandby, etc */
+
     ESD_PROTO_MAX           /* for bounds checking */
 };
+    
 
 /******************/
 /* The EsounD api */
@@ -117,6 +122,9 @@ typedef unsigned char octet;
 /* rate, format = (bits | channels | stream | func) */
 int esd_open_sound( const char *host );
 
+/* send the authorization cookie, create one if needed */
+int esd_send_auth( int sock );
+
 /* lock/unlock will disable/enable foreign clients from connecting */
 int esd_lock( int esd );
 int esd_unlock( int esd );
@@ -217,6 +225,11 @@ typedef struct esd_info {
 
 } esd_info_t;
 
+enum esd_standby_mode { 
+    ESM_ERROR, ESM_ON_STANDBY, ESM_ON_AUTOSTANDBY, ESM_RUNNING
+};
+typedef int esd_standby_mode_t;
+
 /* define callbacks for esd_update_info() */
 /* what to do when a stream connects, or sample is played */
 typedef int esd_new_player_callback_t( esd_player_info_t * );
@@ -269,9 +282,14 @@ int esd_set_stream_pan( int esd, int stream_id,
 int esd_set_default_sample_pan( int esd, int sample_id, 
 				int left_scale, int right_scale );
 
+/* see if the server is in stnaby, autostandby, etc */
+esd_standby_mode_t esd_get_standby_mode( int esd );
+
 
 /*******************************************************************/
 /* esdfile.c - audiofile wrappers for sane handling of files */
+
+int esd_send_file( int esd, AFfilehandle au_file, int frame_length );
 int esd_play_file( const char *name_prefix, const char *filename, int fallback );
 int esd_file_cache( int esd, const char *name_prefix, const char *filename );
 
@@ -280,13 +298,15 @@ int esd_file_cache( int esd, const char *name_prefix, const char *filename );
 /* audio.c - abstract the sound hardware for cross platform usage */
 extern esd_format_t esd_audio_format;
 extern int esd_audio_rate;
+extern char *esd_audio_device;
 
-int esd_audio_open();
-void esd_audio_close();
-void esd_audio_pause();
+const char *esd_audio_devices( void );
+int esd_audio_open( void );
+void esd_audio_close( void );
+void esd_audio_pause( void );
 int esd_audio_write( void *buffer, int buf_size );
 int esd_audio_read( void *buffer, int buf_size );
-void esd_audio_flush();
+void esd_audio_flush( void );
 
 #ifdef __cplusplus
 }
