@@ -1,9 +1,18 @@
 #include "esd.h"
 
 #include <stdio.h>
+#include <unistd.h>
+#include <stdlib.h>
 #include <string.h>
 #include <getopt.h>
 
+/*****************************************************************/
+/* prototypes */
+void exit_usage( const char *who, int errcode, const char *why, const char *what );
+
+
+/*****************************************************************/
+
 void exit_usage( const char *who, int errcode, const char *why, const char *what )
 {
     /* what went wrong? */
@@ -38,6 +47,7 @@ void exit_usage( const char *who, int errcode, const char *why, const char *what
 	     "panstream <id> <left> <right> set panning for a stream\n"
 	     "pansample <id> <left> <right> set default panning for a sample\n"
 	     "                              - left/right pan values scaled to 256.\n"
+	     "standbymode                   see if server's on standby, etc.\n"
 	     "\n" );
 
     /* terminate with given error code */
@@ -51,6 +61,7 @@ int main(int argc, char **argv)
     char *server = NULL;
     esd_server_info_t *server_info = NULL;
     esd_info_t *all_info = NULL;
+    int mode = 0;
   
     struct option opts[] = {
 	{ "server", required_argument, NULL, 's' },
@@ -155,7 +166,7 @@ int main(int argc, char **argv)
 	    left = atoi( argv[ ++optind ] );
 	    right = atoi( argv[ ++optind ] );
 
-	    if ( !id || !left || !right ) {
+	    if ( !id ) {
 		fprintf( stderr, "panstream failed, id = %d, left = %d, right = %d\n",
 			 id, left, right );
 	    } else {
@@ -167,13 +178,35 @@ int main(int argc, char **argv)
 	    left = atoi( argv[ ++optind ] );
 	    right = atoi( argv[ ++optind ] );
 
-	    if ( !id || !left || !right ) {
+	    if ( !id ) {
 		fprintf( stderr, "pansample failed, id = %d, left = %d, right = %d\n",
 			 id, left, right );
 	    } else {
 		esd_set_default_sample_pan( esd, id, left, right );
 	    }
 	}
+	else if ( !strcmp( "standbymode", argv[ optind ] ) ) {
+	    
+	    mode = esd_get_standby_mode( esd );
+	    switch( mode )
+	    {
+	    case ESM_ERROR:
+		printf( "standbymode failed\n" );
+		break;
+	    case ESM_ON_STANDBY:
+		printf( "server is on standby\n" );
+		break;
+	    case ESM_ON_AUTOSTANDBY:
+		printf( "server is on autostandby\n" );
+		break;
+	    case ESM_RUNNING:
+		printf( "server is running\n" );
+		break;
+	    default:
+		printf( "standbymode failed - unexpected return value\n" );
+		break;
+	    };
+	}
 	else {
 	    exit_usage( argv[ 0 ], 3, "unknown command", argv[ optind ] );
 	}
