@@ -1,11 +1,27 @@
-#include "esd-server.h"
+#include "config.h"
+#include "esd.h"
+#include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <dirent.h>
+#include <fcntl.h>
 #include <sys/stat.h>
+#include <time.h>
 
+#include <string.h>
 #include <netdb.h>
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
 #include <arpa/inet.h>
 
+/*******************************************************************/
+/* prototypes */
+int esd_set_socket_buffers( int sock, int src_format, 
+			    int src_rate, int base_rate );
+
+/*******************************************************************/
+/* alternate implementations */
 #ifndef HAVE_INET_ATON
 int inet_aton(const char *cp, struct in_addr *inp)
 {
@@ -31,6 +47,24 @@ int inet_aton(const char *cp, struct in_addr *inp)
 }
 #endif
 
+/*******************************************************************/
+/* set socket buffer lengths to optimal length for audio data transfer */
+int esd_set_socket_buffers( int sock, int src_format, 
+			    int src_rate, int base_rate )
+{
+    int buf_size = ESD_BUF_SIZE;
+
+    if ( src_rate > 0 ) buf_size = ( buf_size * base_rate ) / src_rate;
+    if ( ( src_format & ESD_MASK_BITS ) == ESD_BITS16 )
+	buf_size *= 2;
+    if ( ! ( ( src_format & ESD_MASK_CHAN ) == ESD_MONO ) )
+	buf_size *= 2;
+
+    setsockopt( sock, SOL_SOCKET, SO_SNDBUF, &buf_size, sizeof( buf_size ) );
+    setsockopt( sock, SOL_SOCKET, SO_RCVBUF, &buf_size, sizeof( buf_size ) );
+    return buf_size;
+}
+
 /*******************************************************************/
 /* send the authorization cookie, create one if needed */
 int esd_send_auth( int sock )
@@ -113,6 +147,12 @@ int esd_lock( int esd ) {
     int proto = ESD_PROTO_LOCK;
     int ok = 0;
 
+    /* diagnostic info */
+    /*
+    if ( getenv( "ESDBG" ) )
+        printf( "esound locking\n" );
+    */
+
     write( esd, &proto, sizeof(proto) );
     esd_send_auth( esd );
 
@@ -126,6 +166,12 @@ int esd_unlock( int esd ){
     int proto = ESD_PROTO_UNLOCK;
     int ok = 0;
 
+    /* diagnostic info */
+    /*
+    if ( getenv( "ESDBG" ) )
+	printf( "esound unlocking\n" );
+    */
+
     write( esd, &proto, sizeof(proto) );
     esd_send_auth( esd );
 
@@ -142,6 +188,12 @@ int esd_standby( int esd )
     int proto = ESD_PROTO_STANDBY;
     int ok = 0;
 
+    /* diagnostic info */
+    /*
+    if ( getenv( "ESDBG" ) )
+	printf( "esound standing by\n" );
+    */
+
     write( esd, &proto, sizeof(proto) );
     esd_send_auth( esd );
 
@@ -156,6 +208,12 @@ int esd_resume( int esd )
     int proto = ESD_PROTO_RESUME;
     int ok = 0;
 
+    /* diagnostic info */
+    /*
+    if ( getenv( "ESDBG" ) )
+	printf( "esound resuming\n" );
+    */
+
     write( esd, &proto, sizeof(proto) );
     esd_send_auth( esd );
 
@@ -185,8 +243,14 @@ int esd_open_sound( const char *host )
     char default_host[] = "0.0.0.0";
     char connect_host[64];
     int port = ESD_DEFAULT_PORT;
-    int host_div = 0;
+    unsigned int host_div = 0;
    
+    /* diagnostic info */
+    /*
+    if ( getenv( "ESDBG" ) )
+	printf( "esound opening: %s\n", ( host ? host : "(nil)" ) );
+    */
+
     /* see if we have a remote speaker to play to */
     espeaker = host ? host : getenv( "ESPEAKER" );
     if ( espeaker != NULL ) {
@@ -251,9 +315,9 @@ int esd_open_sound( const char *host )
 		  (struct sockaddr *) &socket_addr,
 		  sizeof(struct sockaddr_in) ) < 0 )
     {
-	fprintf(stderr,"Unable to connect to server port %d\n", 
-		socket_addr.sin_port );
-	fprintf(stderr,"This generally means that the program could not talk to the esound server\n");
+	/* fprintf(stderr,"Unable to connect to EsounD server at port %d\n", 
+		port ); */
+	/* fprintf(stderr,"This generally means that the program could not talk to the esound server\n"); */
 	return -1;
     }
 
@@ -261,9 +325,22 @@ int esd_open_sound( const char *host )
     if ( !esd_send_auth( socket_out ) ) {
 	/* couldn't send authorization key, bail */
 	close( socket_out );
+
+	/* diagnostic info */
+	/*
+	if ( getenv( "ESDBG" ) )
+	    printf( "esound opening: authorization failed\n" );
+	*/
+
 	return -1;
     }
 
+    /* diagnostic info */
+    /*
+    if ( getenv( "ESDBG" ) )
+	printf( "esound opening: assigned to %d\n", socket_out );
+    */
+
     return socket_out;
 }
 
@@ -297,33 +374,18 @@ int esd_play_stream( esd_format_t format, int rate,
     if( write( sock, name_buf, ESD_NAME_MAX ) != ESD_NAME_MAX )
 	return -1;
 
-  /* Reduce buffers on sockets to the minimum needed */
-    {
-      int fd_bufsize;
-      int src_format;
-      
-      src_format = format;
-      fd_bufsize = ESD_BUF_SIZE;
-      if (rate > 0)
-	fd_bufsize = (fd_bufsize * 44100) / rate;
-      else
-	fd_bufsize = (fd_bufsize * 44100) / 44100;
-      if ((src_format & ESD_MASK_BITS) == ESD_BITS16)
-	{
-	  fd_bufsize *= 2;
-	}
-      if (!((src_format & ESD_MASK_CHAN) == ESD_MONO))
-	{
-	  fd_bufsize *= 2;	      
-	}
-      setsockopt(sock, SOL_SOCKET, SO_SNDBUF, 
-		 &fd_bufsize, sizeof(fd_bufsize));
-      setsockopt(sock, SOL_SOCKET, SO_RCVBUF,  
-		 &fd_bufsize, sizeof(fd_bufsize));
-    }
+    /* Reduce buffers on sockets to the minimum needed */
+    esd_set_socket_buffers( sock, format, rate, 44100 );
+
     /* flush the socket */
     /* fsync( sock ); */
     
+    /* diagnostic info */
+    /*
+    if ( getenv( "ESDBG" ) )
+	printf( "esound playing stream\n" );
+    */
+
     return sock;
 }
 
@@ -347,6 +409,12 @@ int esd_play_stream_fallback( esd_format_t format, int rate,
     esd_audio_rate = rate;
     socket_out = esd_audio_open();
 
+    /* diagnostic info */
+    /*
+    if ( getenv( "ESDBG" ) )
+	printf( "esound playing stream fallback\n" );
+    */
+
     /* we either got it, or we didn't */
     return socket_out;
 }
@@ -381,33 +449,18 @@ int esd_monitor_stream( esd_format_t format, int rate,
     if( write( sock, name_buf, ESD_NAME_MAX ) != ESD_NAME_MAX )
 	return -1;
 
-  /* Reduce buffers on sockets to the minimum needed */
-    {
-      int fd_bufsize;
-      int src_format;
-      
-      src_format = format;
-      fd_bufsize = ESD_BUF_SIZE;
-      if (rate > 0)
-	fd_bufsize = (fd_bufsize * 44100) / rate;
-      else
-	fd_bufsize = (fd_bufsize * 44100) / 44100;
-      if ((src_format & ESD_MASK_BITS) == ESD_BITS16)
-	{
-	  fd_bufsize *= 2;
-	}
-      if (!((src_format & ESD_MASK_CHAN) == ESD_MONO))
-	{
-	  fd_bufsize *= 2;	      
-	}
-      setsockopt(sock, SOL_SOCKET, SO_SNDBUF, 
-		 &fd_bufsize, sizeof(fd_bufsize));
-      setsockopt(sock, SOL_SOCKET, SO_RCVBUF,  
-		 &fd_bufsize, sizeof(fd_bufsize));
-    }
+    /* Reduce buffers on sockets to the minimum needed */
+    esd_set_socket_buffers( sock, format, rate, 44100 );
+
     /* flush the socket */
     /* fsync( sock ); */
     
+    /* diagnostic info */
+    /*
+    if ( getenv( "ESDBG" ) )
+	printf( "esound monitoring stream\n" );
+    */
+
     return sock;
 }
 
@@ -441,33 +494,18 @@ int esd_filter_stream( esd_format_t format, int rate,
     if( write( sock, name_buf, ESD_NAME_MAX ) != ESD_NAME_MAX )
 	return -1;
 
-  /* Reduce buffers on sockets to the minimum needed */
-    {
-      int fd_bufsize;
-      int src_format;
-      
-      src_format = format;
-      fd_bufsize = ESD_BUF_SIZE;
-      if (rate > 0)
-	fd_bufsize = (fd_bufsize * 44100) / rate;
-      else
-	fd_bufsize = (fd_bufsize * 44100) / 44100;
-      if ((src_format & ESD_MASK_BITS) == ESD_BITS16)
-	{
-	  fd_bufsize *= 2;
-	}
-      if (!((src_format & ESD_MASK_CHAN) == ESD_MONO))
-	{
-	  fd_bufsize *= 2;	      
-	}
-      setsockopt(sock, SOL_SOCKET, SO_SNDBUF, 
-		 &fd_bufsize, sizeof(fd_bufsize));
-      setsockopt(sock, SOL_SOCKET, SO_RCVBUF,  
-		 &fd_bufsize, sizeof(fd_bufsize));
-    }
+    /* Reduce buffers on sockets to the minimum needed */
+    esd_set_socket_buffers( sock, format, rate, 44100 );
+
     /* flush the socket */
     /* fsync( sock ); */
     
+    /* diagnostic info */
+    /*
+    if ( getenv( "ESDBG" ) )
+	printf( "esound filterng stream\n" );
+    */
+
     return sock;
 }
 
@@ -501,33 +539,18 @@ int esd_record_stream( esd_format_t format, int rate,
     if( write( sock, name_buf, ESD_NAME_MAX ) != ESD_NAME_MAX )
 	return -1;
 
-  /* Reduce buffers on sockets to the minimum needed */
-    {
-      int fd_bufsize;
-      int src_format;
-      
-      src_format = format;
-      fd_bufsize = ESD_BUF_SIZE;
-      if (rate > 0)
-	fd_bufsize = (fd_bufsize * 44100) / rate;
-      else
-	fd_bufsize = (fd_bufsize * 44100) / 44100;
-      if ((src_format & ESD_MASK_BITS) == ESD_BITS16)
-	{
-	  fd_bufsize *= 2;
-	}
-      if (!((src_format & ESD_MASK_CHAN) == ESD_MONO))
-	{
-	  fd_bufsize *= 2;	      
-	}
-      setsockopt(sock, SOL_SOCKET, SO_SNDBUF, 
-		 &fd_bufsize, sizeof(fd_bufsize));
-      setsockopt(sock, SOL_SOCKET, SO_RCVBUF,  
-		 &fd_bufsize, sizeof(fd_bufsize));
-    }
+    /* Reduce buffers on sockets to the minimum needed */
+    esd_set_socket_buffers( sock, format, rate, 44100 );
+
     /* flush the socket */
     /* fsync( sock ); */
     
+    /* diagnostic info */
+    /*
+    if ( getenv( "ESDBG" ) )
+	printf( "esound recording stream\n" );
+    */
+
     return sock;
 }
 
@@ -551,30 +574,15 @@ int esd_record_stream_fallback( esd_format_t format, int rate,
     esd_audio_format = format;
     esd_audio_rate = rate;
     socket_out = esd_audio_open();
-  /* Reduce buffers on sockets to the minimum needed */
-    {
-      int fd_bufsize;
-      int src_format;
-      
-      src_format = format;
-      fd_bufsize = ESD_BUF_SIZE;
-      if (rate > 0)
-	fd_bufsize = (fd_bufsize * 44100) / rate;
-      else
-	fd_bufsize = (fd_bufsize * 44100) / 44100;
-      if ((src_format & ESD_MASK_BITS) == ESD_BITS16)
-	{
-	  fd_bufsize *= 2;
-	}
-      if (!((src_format & ESD_MASK_CHAN) == ESD_MONO))
-	{
-	  fd_bufsize *= 2;	      
-	}
-      setsockopt(socket_out, SOL_SOCKET, SO_SNDBUF, 
-		 &fd_bufsize, sizeof(fd_bufsize));
-      setsockopt(socket_out, SOL_SOCKET, SO_RCVBUF,  
-		 &fd_bufsize, sizeof(fd_bufsize));
-    }
+
+    /* Reduce buffers on sockets to the minimum needed */
+    esd_set_socket_buffers( socket_out, format, rate, 44100 );
+
+    /* diagnostic info */
+    /*
+    if ( getenv( "ESDBG" ) )
+	printf( "esound recording stream fallback\n" );
+    */
 
     /* we either got it, or we didn't */
     return socket_out;
@@ -617,6 +625,12 @@ int esd_sample_cache( int esd, esd_format_t format, int rate,
     if ( read( esd, &id, sizeof(id) ) != sizeof(id) )
 	return -1;
 
+    /* diagnostic info */
+    /*
+    if ( getenv( "ESDBG" ) )
+	printf( "esound caching sample\n" );
+    */
+
     /* return the sample id to the client */
     return id;
 }
@@ -632,6 +646,12 @@ int esd_confirm_sample_cache( int esd )
     if ( read( esd, &id, sizeof(id) ) != sizeof(id) )
 	return -1;
 
+    /* diagnostic info */
+    /*
+    if ( getenv( "ESDBG" ) )
+	printf( "esound confirming cached sample\n" );
+    */
+
     /* return the sample id to the client */
     return id;
 }
@@ -663,6 +683,13 @@ int esd_sample_getid( int esd, const char *name)
     if ( read( esd, &id, sizeof(id) ) != sizeof(id) )
 	return -1;
 
+    /* diagnostic info */
+    /*
+    if ( getenv( "ESDBG" ) )
+	printf( "esound getting cached sample id: \'%s\' = %d\n",
+		name, id );
+    */
+    
     /* return the sample id to the client */
     return id;
 }
@@ -687,6 +714,12 @@ int esd_sample_free( int esd, int sample )
     if ( read( esd, &id, sizeof(id) ) != sizeof(id) )
 	return -1;
 
+    /* diagnostic info */
+    /*
+    if ( getenv( "ESDBG" ) )
+        printf( "esound freeing sample\n" );
+    */
+
     /* return the id to the client (0 = error, 1 = ok) */
     return id;
 }
@@ -711,6 +744,12 @@ int esd_sample_play( int esd, int sample )
     if ( read( esd, &is_ok, sizeof(is_ok) ) != sizeof(is_ok) )
 	return -1;
 
+    /* diagnostic info */
+    /*
+    if ( getenv( "ESDBG" ) )
+	printf( "esound playing sample\n" );
+    */
+
     /* return the id to the client (0 = error, 1 = ok) */
     return is_ok;
 }
@@ -736,6 +775,12 @@ int esd_sample_loop( int esd, int sample )
     if ( read( esd, &is_ok, sizeof(is_ok) ) != sizeof(is_ok) )
 	return -1;
 
+    /* diagnostic info */
+    /*
+    if ( getenv( "ESDBG" ) )
+	printf( "esound looping sample\n" );
+    */
+
     /* return the id to the client (0 = error, 1 = ok) */
     return is_ok;
 }
@@ -760,6 +805,12 @@ int esd_sample_stop( int esd, int sample )
     if ( read( esd, &is_ok, sizeof(is_ok) ) != sizeof(is_ok) )
 	return -1;
 
+    /* diagnostic info */
+    /*
+    if ( getenv( "ESDBG" ) )
+	printf( "esound stopping sample\n" );
+    */
+
     /* return the id to the client (0 = error, 1 = ok) */
     return is_ok;
 }
@@ -768,5 +819,11 @@ int esd_sample_stop( int esd, int sample )
 /* closes fd, previously obtained by esd_open */
 int esd_close( int esd )
 {
+    /* diagnostic info */
+    /*
+    if ( getenv( "ESDBG" ) )
+	printf( "esound closing\n" );
+    */
+
     return close( esd );
 }
