@@ -4,11 +4,17 @@
 #include <stdlib.h>
 #include <dirent.h>
 #include <signal.h>
+#include <string.h>
+#include <unistd.h>
 #include <sys/stat.h>
 
+/* prototype(s) */
+void clean_exit(int signum);
+
 volatile int terminate = 0;
 
 void clean_exit(int signum) {
+    fprintf( stderr, "received signal %d: terminating...\n", signum );
     terminate = 1;
     return;
 }
@@ -29,7 +35,7 @@ int main(int argc, char **argv)
     char *host = NULL;
     char *name = NULL;
     char filename[ ESD_NAME_MAX ] = "";
-    int cache_mode = 0; /* cache mode, 1 = data, 0 = file */
+    int cache_mode = 0; /* cache mode, 1 = data, 0 = file, 2 = existing */
     
     for ( arg = 1 ; arg < argc ; arg++)
     {
@@ -47,6 +53,8 @@ int main(int argc, char **argv)
 	    channels = ESD_MONO;
 	else if ( !strcmp( "-d", argv[ arg ] ) )
 	    cache_mode = 1;
+	else if ( !strcmp( "-e", argv[ arg ] ) )
+	    cache_mode = 2;
 	else if ( !strcmp( "-r", argv[ arg ] ) )
 	{
 	    arg++;
@@ -64,14 +72,13 @@ int main(int argc, char **argv)
 	     ESD_NAME_MAX - strlen( filename ) );
     name = argv[ arg ];
 
-printf( "name is \'%s\'.\n", filename );
 
     /* if we see any of these, terminate */
     signal( SIGINT, clean_exit );
     signal( SIGKILL, clean_exit );
     signal( SIGPIPE, clean_exit );
 
-    if ( cache_mode ) {
+    if ( cache_mode == 1 ) {
 	source = fopen( name, "r" );
 
 	if ( source == NULL ) {
@@ -110,7 +117,7 @@ printf( "name is \'%s\'.\n", filename );
 
 	printf( "sample <%d> uploaded, %d bytes\n", sample_id, total );
     } 
-    else {
+    else if ( cache_mode == 0) {
 	sock = esd_open_sound( host );
 	if ( sock <= 0 ) 
 	    return 1;
@@ -120,19 +127,24 @@ printf( "name is \'%s\'.\n", filename );
     
 	/* if we see any of these, terminate */
 	if ( sample_id < 0 ) {
-	    printf( "error while caching sample <%d>: confirm value != sample_id, %d\n",
+	    printf( "error while caching sample <%d>: confirm value != sample_id\n",
 		    sample_id );
 	    exit( 1 );
 	}
 
 	printf( "sample <%d> uploaded: %s\n", sample_id, filename );
+    } else if (cache_mode == 2) {
+	strcpy(filename, name);
     }
 
+    printf( "name is \'%s\'.\n", filename );
+
     reget_sample_id = esd_sample_getid( sock, filename );
 
-    printf( "reget of sample id is <%d>\n", reget_sample_id );
-    if( reget_sample_id != sample_id ) {
-	printf( "sample id's do not match!\n" );
+    printf( "reget of sample %s id is <%d>\n", filename, reget_sample_id );
+    if(( reget_sample_id != sample_id && cache_mode != 2)
+       || reget_sample_id < 0) {
+	printf( "sample id's do not make sense! (cache mode %d)\n", cache_mode );
 	exit( 1 );
     }
 
