@@ -1,5 +1,6 @@
 
 #include "esd-server.h"
+#include <errno.h>
 
 /*******************************************************************/
 /* globals */
@@ -36,16 +37,17 @@ void free_player( esd_player_t *player )
     } else if ( ( player->format & ESD_MASK_MODE ) == ESD_SAMPLE ) {
 	sample = (esd_sample_t *) (player->parent);
 	sample->ref_count--;
+	esd_playing_samples--;
 
-	ESDBG_TRACE( printf( "<%02d> free player: [%p] refs=%d erase?=%d\n", 
+	ESDBG_TRACE( printf( "<%02d> free player: [%p] refs=%d erase?=%d samps=%d\n", 
 			     player->source_id, player, sample->ref_count, 
-			     sample->erase_when_done ); );
+			     sample->erase_when_done, esd_playing_samples ); );
 
 	if ( sample->erase_when_done && !sample->ref_count ) {
 	    ESDBG_TRACE( printf( "<%02d> free_player: erasing sample\n", 
 				 sample->sample_id ); );
 
-	    erase_sample( sample->sample_id );
+	    erase_sample( sample->sample_id, 0 );
 	}
     }
 
@@ -63,6 +65,11 @@ void free_player( esd_player_t *player )
 void add_player( esd_player_t *player )
 {
     /* printf ( "adding player %p\n", new_player ); */
+    if ( !player ) {
+	ESDBG_TRACE( printf( "<NIL> can't add non-existent player!\n" ); );
+	return;
+    }
+
     player->next = esd_players_list;
     esd_players_list = player;
     return;
@@ -143,6 +150,74 @@ void erase_monitor( esd_player_t *monitor )
 }
 
 
+/*******************************************************************/
+/* write block of data from client, return < 0 to have it erased */
+int write_player( esd_player_t *player, void *src_buffer, int src_length, 
+		  int src_rate, int src_format )
+{
+    fd_set wr_fds;
+    int length = 0, actual = 0, can_write = 0;
+    struct timeval timeout;
+    char message[ 100 ];
+    short data, *pos; /* used for swapping */
+    esd_client_t *client;
+    short *buffer;
+
+    /* use select to prevent blocking clients that are ready */
+    timeout.tv_sec = 0;
+    timeout.tv_usec = 0;
+    FD_ZERO( &wr_fds );
+    FD_SET( player->source_id, &wr_fds );
+    
+    /* if the data is ready, read a block */
+    can_write = select( player->source_id + 1, 
+			NULL, &wr_fds, NULL, &timeout ) ;
+    if ( can_write > 0 )
+    {
+	/* translate the data */
+	length = player->translate_func( player->data_buffer, 
+					 player->buffer_length, 
+					 player->rate, 
+					 player->format, 
+					 src_buffer, 
+					 src_length, 
+					 src_rate, 
+					 src_format );
+
+	/* endian swap multi-byte data if we need to */
+	client = (esd_client_t *) (player->parent);
+	if ( client->swap_byte_order 
+	     && ( (player->format & ESD_MASK_BITS) == ESD_BITS16 ) )
+	{
+	    printf( "swapping...\n" );
+	    buffer = (short*) player->data_buffer;
+	    for ( pos = buffer 
+		      ; pos < buffer + length / sizeof(short)
+		      ; pos += sizeof(short) )
+	    {
+		data = swap_endian_16( (*pos) );
+		*pos = data;
+	    }
+	}
+
+	/* write out the data */
+	ESD_WRITE_BIN( player->source_id, player->data_buffer, 
+		       player->buffer_length, length, "str rd" );
+
+    } else if ( can_write < 0 ) {
+	sprintf( message, "error writing client (%d)\n", 
+		 player->source_id );
+	perror( message );
+	return -1;
+    }
+
+    /* check for end of stream */
+    if ( length < 0 ) return -1;
+    if ( length == 0 ) return 0;
+ 
+    return actual;
+}
+
 /*******************************************************************/
 /* read block of data from client, return < 0 to have it erased */
 int read_player( esd_player_t *player )
@@ -172,7 +247,8 @@ int read_player( esd_player_t *player )
 			  player->buffer_length, actual, "str rd" );
 
 	    /* check for end of stream */
-	    if ( actual == 0 ) 
+	    if ( actual == 0 
+		 || ( actual < 0 && errno != EAGAIN && errno != EINTR ) )
 		return -1;
  
 	    /* more data, save how much we got */
@@ -303,16 +379,14 @@ void monitor_write( void *output_buffer, int length ) {
 	    break;
 
 	/* mix down the monitor's buffer */
-	if ( (esd_audio_format & ESD_MASK_BITS) == ESD_BITS16 ) {
-	    length = mix_from_stereo_16s( monitor->data_buffer, monitor->buffer_length,
-					  monitor->rate, monitor->format, 
-					  output_buffer, length, esd_audio_rate );
-	}
-	else {
-	    length = mix_from_stereo_8u( monitor->data_buffer, monitor->buffer_length,
-					 monitor->rate, monitor->format, 
-					 output_buffer, length, esd_audio_rate );
-	}
+	length = monitor->translate_func( monitor->data_buffer, 
+					  monitor->buffer_length,
+					  monitor->rate, 
+					  monitor->format, 
+					  output_buffer, 
+					  length, 
+					  esd_audio_rate,
+					  esd_audio_format );
 
 	/* write the data buffer to the socket */
 	ESD_WRITE_BIN( monitor->source_id, monitor->data_buffer, 
@@ -336,31 +410,20 @@ void monitor_write( void *output_buffer, int length ) {
     return;
 }
 
-void recorder_write() {
-    fd_set wr_fds;
-    int length, can_write;
-    struct timeval timeout;
+int recorder_write( void *buffer, int length ) {
 
-    /* make sure we have a recorder connected */
-    if ( !esd_recorder ) 
-	return;
-
-    /* see if we can write to the socket */
-    timeout.tv_sec = 0;
-    timeout.tv_usec = 0;
-    FD_ZERO( &wr_fds );
-    FD_SET( esd_recorder->source_id, &wr_fds );
-    can_write = select( esd_recorder->source_id + 1, NULL, &wr_fds, NULL, &timeout );
-    if ( !can_write ) 
-	return;
-
-    /* write the data buffer to the socket */
-    ESD_WRITE_BIN( esd_recorder->source_id, esd_recorder->data_buffer, 
-		   esd_recorder->buffer_length, length, "rec wr" );
+    /* write it out */
+    ESDBG_TRACE( printf( "(%02d) writing recorder data\n", 
+			 esd_recorder->source_id ); );
+    length = write_player( esd_recorder, buffer,  length, 
+			   esd_audio_rate, esd_audio_format);
 
+    /* see how it went */
     if ( length < 0 ) {
+
 	/* couldn't send anything, close it down */
-	ESDBG_TRACE( printf( "(%02d) closing recorder\n", esd_recorder->source_id ); );
+	ESDBG_TRACE( printf( "(%02d) closing recorder\n", 
+			     esd_recorder->source_id ); );
 
 	/* stop recording */
 	esd_audio_close();
@@ -373,7 +436,7 @@ void recorder_write() {
 	esd_recorder = NULL;
     }
 
-    return;
+    return length;
 }
 
 /*******************************************************************/
@@ -381,7 +444,6 @@ void recorder_write() {
 esd_player_t *new_stream_player( esd_client_t *client )
 {
     esd_player_t *player;
-    int actual;
 
     /* make sure we have the memory to save the client... */
     player = (esd_player_t*) malloc( sizeof(esd_player_t) );
@@ -393,50 +455,24 @@ esd_player_t *new_stream_player( esd_client_t *client )
     player->next = NULL;
     player->parent = NULL;
 
-    ESD_READ_INT( client->fd, &player->format, sizeof(player->format), 
-		  actual, "newstr fmt" );
-    if ( client->swap_byte_order )
-	player->format = swap_endian_32( player->format );
+    player->format = *(int*)(client->proto_data);
+    player->rate = *(int*)(client->proto_data + sizeof(int));
 
+    player->format = maybe_swap_32( client->swap_byte_order, player->format );
     player->format &= ~ESD_MASK_MODE; /* force to ESD_STREAM */
-    ESD_READ_INT( client->fd, &player->rate, sizeof(player->rate), 
-		  actual, "newstr rat" );
-    if ( client->swap_byte_order )
-	player->rate = swap_endian_32( player->rate );
+    player->rate = maybe_swap_32( client->swap_byte_order, player->rate );
 
     player->source_id = client->fd;
-    ESD_READ_BIN( client->fd, player->name, ESD_NAME_MAX, 
-		  actual, "newstr nam" );
-
+    strncpy( player->name, client->proto_data + 2 * sizeof(int), ESD_NAME_MAX );
     player->name[ ESD_NAME_MAX - 1 ] = '\0';
 
     ESDBG_TRACE( printf( "(%02d) stream %s: 0x%08x at %d Hz\n", client ->fd, 
 			 player->name, player->format, player->rate ); );
 
-  /* Reduce buffers on sockets to the minimum needed */
-    {
-      int fd_bufsize;
-      int src_format;
-      
-      src_format = player->format;
-      fd_bufsize = esd_buf_size_octets;
-      if (player->rate > 0)
-	fd_bufsize = (fd_bufsize * esd_audio_rate) / player->rate;
-      else
-	fd_bufsize = (fd_bufsize * esd_audio_rate) / 44100;
-      if ((src_format & ESD_MASK_BITS) == ESD_BITS16)
-	{
-	  fd_bufsize *= 2;
-	}
-      if (!((src_format & ESD_MASK_CHAN) == ESD_MONO))
-	{
-	  fd_bufsize *= 2;	      
-	}
-      setsockopt(player->source_id, SOL_SOCKET, SO_SNDBUF, 
-		 &fd_bufsize, sizeof(fd_bufsize));
-      setsockopt(player->source_id, SOL_SOCKET, SO_RCVBUF,  
-		 &fd_bufsize, sizeof(fd_bufsize));
-    }
+    /* Reduce buffers on sockets to the minimum needed */
+    esd_set_socket_buffers( player->source_id, player->format, 
+			    player->rate, esd_audio_rate );
+
     /* calculate buffer length to match the mix buffer duration */
     player->buffer_length = esd_buf_size_octets * player->rate / esd_audio_rate;
     if ( (player->format & ESD_MASK_BITS) == ESD_BITS8 )
@@ -456,9 +492,10 @@ esd_player_t *new_stream_player( esd_client_t *client )
 	return NULL;
     }
 
-    /* everything's ok, return the allocated player */
-    /* player->last_read = time(NULL); */
+    /* everything's ok, set the easy stuff */
     player->left_vol_scale = player->right_vol_scale = ESD_VOLUME_BASE;
+    player->mix_func = get_mix_func( player );
+    player->translate_func = NULL; /* no translating, just mixing */
 
     ESDBG_TRACE( printf( "(%02d) player: [%p]\n", player->source_id, player ); );
 
@@ -527,13 +564,17 @@ esd_player_t *new_sample_player( int sample_id, int loop )
 	return NULL;
     }
 
-    /* everything's ok, return the allocated player */
+    /* update housekeeping values */
+    esd_playing_samples++;
     player->last_pos = 0;
     sample->ref_count++;
-    sample->erase_when_done = 0;
+    player->mix_func = get_mix_func( player );
+    player->translate_func = NULL; /* no translating, just mixing */
 
-    ESDBG_TRACE( printf( "<%02d> new player: refs=%d [%p]\n", 
-			 player->source_id, sample->ref_count, player ); );
+    ESDBG_TRACE( printf( "<%02d> new player: refs=%d samps=%d [%p]\n", 
+			 player->source_id, sample->ref_count, 
+			 esd_playing_samples, player ); );
 
+    /* everything's ok, return the allocated player */
     return player;
 }
