@@ -1,11 +1,18 @@
 
 #include "esd-server.h"
+#include <errno.h>
 
 /*******************************************************************/
 /* globals */
 esd_sample_t *esd_samples_list = NULL;
 
-static int esd_next_sample_id = 1;	/* sample id = 0 is an error */
+int esd_playing_samples = 0;
+int esd_next_sample_id = 1;	/* sample id = 0 is an error */
+
+/*******************************************************************/
+/* prototypes */
+void free_sample( esd_sample_t *sample );
+void dump_samples(void);
 
 /*******************************************************************/
 /* for debugging purposes, dump the list of the clients and data */
@@ -31,8 +38,6 @@ void free_sample( esd_sample_t *sample )
     /* free any memory allocated with the sample */
     free( sample->data_buffer );
 
-    /* TODO: free sample players with this id */
-
     /* free the sample memory itself */
     free( sample );
     ESDBG_TRACE( printf( "<%02d> freed sample: [0x%p]\n", 
@@ -42,18 +47,23 @@ void free_sample( esd_sample_t *sample )
 
 /*******************************************************************/
 /* add a complete new client into the list of samples at head */
-void add_sample( esd_sample_t *new_sample )
+void add_sample( esd_sample_t *sample )
 {
     /* printf ( "adding sample 0x%p\n", new_sample ); */
-    new_sample->next = esd_samples_list;
-    esd_samples_list = new_sample;
+    if ( !sample ) {
+	ESDBG_TRACE( printf( "<NIL> can't add non-existent sample!\n" ); );
+	return;
+    }
+
+    sample->next = esd_samples_list;
+    esd_samples_list = sample;
     return;
 }
 
 /*******************************************************************/
 /* erase a sample from the sample list */
 /* TODO: add "force kill" boolean option */
-void erase_sample( int id )
+void erase_sample( int id, int force )
 {
     esd_sample_t *previous = NULL;
     esd_sample_t *current = esd_samples_list;
@@ -67,7 +77,7 @@ void erase_sample( int id )
 	if ( current->sample_id == id ) {
 
 	    /* if the ref count is non-zero, just flag it for deletion */
-	    if ( current->ref_count ) {
+	    if ( current->ref_count && !force ) {
 		ESDBG_TRACE( printf( "<%02d> erasing sample - deferred\n", id ); );
 		current->erase_when_done = 1;
 		return;
@@ -115,6 +125,10 @@ int read_sample( esd_sample_t *sample )
 		      min( esd_buf_size_octets, sample->sample_length-total),
 		      actual, "rd_samp" );
 
+	if ( actual == 0 
+	     || ( actual < 0 && errno != EAGAIN && errno != EINTR ) )
+	    return 0;
+
 	if ( actual > 0 ) {
 	    /* endian swap multi-byte data if we need to */
 	    if ( client->swap_byte_order 
@@ -138,7 +152,7 @@ int read_sample( esd_sample_t *sample )
     /* TODO: what if total != sample_length ? */
     ESDBG_TRACE( printf( "<%02d> %d bytes total\n", sample->sample_id, total ); );
     sample->cached_length = total;
-    return total;
+    return 1;
 }
 
 /*******************************************************************/
@@ -157,22 +171,17 @@ esd_sample_t *new_sample( esd_client_t *client )
     /* and initialize the sample */
     sample->next = NULL;
     sample->parent = NULL;
-    ESD_READ_INT( client->fd, &sample->format, sizeof(sample->format),
-		  actual, "ns fmt" );
-    if ( client->swap_byte_order )
-	sample->format = swap_endian_32( sample->format );
 
-    ESD_READ_INT( client->fd, &sample->rate, sizeof(sample->rate),
-		  actual, "ns rate" );
-    if ( client->swap_byte_order )
-	sample->rate = swap_endian_32( sample->rate );
+    sample->format = *(int*)(client->proto_data);
+    sample->rate = *(int*)(client->proto_data + sizeof(int));
+    sample->sample_length = *(int*)(client->proto_data + 2 * sizeof(int));
 
-    ESD_READ_INT( client->fd, &sample->sample_length, sizeof(sample->sample_length),
-		  actual, "ns len" );
-    if ( client->swap_byte_order )
-	sample->sample_length = swap_endian_32( sample->sample_length );
+    sample->format = maybe_swap_32( client->swap_byte_order, sample->format );
+    sample->rate = maybe_swap_32( client->swap_byte_order, sample->rate );
+    sample->sample_length = maybe_swap_32( client->swap_byte_order, 
+					   sample->sample_length );
 
-    ESD_READ_BIN( client->fd, sample->name, ESD_NAME_MAX, actual, "ns name" );
+    strncpy( sample->name, client->proto_data + 3 * sizeof(int), ESD_NAME_MAX );
     sample->name[ ESD_NAME_MAX - 1 ] = '\0';
 
     sample->sample_id = esd_next_sample_id++;
@@ -193,15 +202,17 @@ esd_sample_t *new_sample( esd_client_t *client )
     /* set ref. count, cached_length, and other housekeeping values */
     sample->cached_length = 0;
     sample->ref_count = 0;
+    sample->erase_when_done = 0;
     sample->left_vol_scale = sample->right_vol_scale = ESD_VOLUME_BASE;
 
-    ESDBG_TRACE( printf( "<%02d> sample %s: [0x%p] - %d bytes\n", sample->sample_id, 
-			 sample->name, sample, sample->sample_length ); );
+    ESDBG_TRACE( printf( "<%02d> sample %s: [0x%p] - %d bytes\n", 
+			 sample->sample_id, 
+			 sample->name, sample, 
+			 sample->sample_length ); );
 
     client_id = maybe_swap_32( client->swap_byte_order, 
 			       sample->sample_id );
     ESD_WRITE_INT( client->fd, &client_id, sizeof(client_id), actual, "ns ack" );
-    /* fsync( client->fd ); */
 
     return sample;
 }
